@@ -55,4 +55,6 @@ from . import (
     dots,
     genome,
     sample,
+    coverage,
+    virtual4c,
 )
