@@ -104,12 +104,16 @@ def test_pileup(request):
             "end": [107_000_000, 113_000_000],
         }
     )
-    stack = cooltools.api.snipping.pileup(clr, windows, view_df, exp, flank=None)
+
+    stack = cooltools.api.snipping.pileup(
+        clr, windows, view_df=view_df, expected_df=exp, flank=None
+    )
+
     # Check that the size of snips is OK and there are two of them:
     assert stack.shape == (5, 5, 2)
 
     # II.
-    # Example off-diagonal features, two regions from annotated genomic regions:
+    # Example off-diagonal features, two features from annotated genomic regions:
     windows = pd.DataFrame(
         {
             "chrom1": ["chr1", "chr1"],
@@ -120,7 +124,9 @@ def test_pileup(request):
             "end2": [112_000_000, 118_000_000],
         }
     )
-    stack = cooltools.api.snipping.pileup(clr, windows, view_df, exp, flank=None)
+    stack = cooltools.api.snipping.pileup(
+        clr, windows, view_df=view_df, expected_df=exp, flank=None
+    )
     # Check that the size of snips is OK and there are two of them:
     assert stack.shape == (5, 5, 2)
 
@@ -136,7 +142,9 @@ def test_pileup(request):
             "end2": [110_000_000, 115_000_000],
         }
     )
-    stack = cooltools.api.snipping.pileup(clr, windows, view_df, exp, flank=None)
+    stack = cooltools.api.snipping.pileup(
+        clr, windows, view_df=view_df, expected_df=exp, flank=None
+    )
     # Check that the size of snips is OK and there are two of them:
     assert stack.shape == (5, 5, 2)
 
@@ -172,7 +180,9 @@ def test_pileup(request):
         }
     )
     with pytest.raises(ValueError):
-        stack = cooltools.api.snipping.pileup(clr, windows, view_df, exp, flank=None)
+        stack = cooltools.api.snipping.pileup(
+            clr, windows, view_df=view_df, expected_df=exp, flank=None
+        )
 
     # DRAFT # Should work with force=True:
     # stack = cooltools.api.snipping.pileup(clr, windows, view_df, exp, flank=0, force=True)
@@ -180,7 +190,7 @@ def test_pileup(request):
     # assert stack.shape == (5, 5, 2)
 
 
-def test_ondiag_pileup_legacy_with_expected(request):
+def test_ondiag__pileup_with_expected(request):
     """
     Test the snipping on matrix:
     """
@@ -201,10 +211,10 @@ def test_ondiag_pileup_legacy_with_expected(request):
         windows = cooltools.api.snipping.make_bin_aligned_windows(
             1_000_000, ["chr1", "chr1"], [102_000_000, 105_000_000], flank_bp=2_000_000
         )
-        windows = cooltools.api.snipping.assign_regions(windows, view_df).reset_index(
+        windows = cooltools.lib.common.assign_view_auto(windows, view_df).reset_index(
             drop=True
         )
-        stack = cooltools.api.snipping.pileup_legacy(
+        stack = cooltools.api.snipping._pileup(
             windows, snipper.select, snipper.snip, map=map
         )
 
@@ -216,11 +226,11 @@ def test_ondiag_pileup_legacy_with_expected(request):
         windows = cooltools.api.snipping.make_bin_aligned_windows(
             1_000_000, ["chr1", "chr1"], [120_000_000, 160_000_000], flank_bp=2_000_000
         )
-        windows = cooltools.api.snipping.assign_regions(windows, view_df).reset_index(
+        windows = cooltools.lib.common.assign_view_auto(windows, view_df).reset_index(
             drop=True
         )
 
-        stack = cooltools.api.snipping.pileup_legacy(
+        stack = cooltools.api.snipping._pileup(
             windows, snipper.select, snipper.snip, map=map
         )
 
@@ -228,7 +238,7 @@ def test_ondiag_pileup_legacy_with_expected(request):
         assert np.all(np.isnan(stack[:, :, 1]))
 
 
-def test_ondiag_pileup_legacy_without_expected(request):
+def test_ondiag__pileup_without_expected(request):
     """
     Test the snipping on matrix:
     """
@@ -244,12 +254,11 @@ def test_ondiag_pileup_legacy_without_expected(request):
         1_000_000, ["chr1", "chr1"], [120_000_000, 160_000_000], flank_bp=2_000_000
     )
 
-    windows = cooltools.api.snipping.assign_regions(windows, view_df).reset_index(
+    windows = cooltools.lib.common.assign_view_auto(windows, view_df).reset_index(
         drop=True
     )
-
     snipper = cooltools.api.snipping.CoolerSnipper(clr, view_df=view_df, min_diag=None)
-    stack = cooltools.api.snipping.pileup_legacy(
+    stack = cooltools.api.snipping._pileup(
         windows, snipper.select, snipper.snip, map=map
     )
 
@@ -261,11 +270,11 @@ def test_ondiag_pileup_legacy_without_expected(request):
     windows = cooltools.api.snipping.make_bin_aligned_windows(
         1_000_000, ["chr1", "chr1"], [120_000_000, 160_000_000], flank_bp=2_000_000
     )
-    windows = cooltools.api.snipping.assign_regions(windows, view_df).reset_index(
+    windows = cooltools.lib.common.assign_view_auto(windows, view_df).reset_index(
         drop=True
     )
 
-    stack = cooltools.api.snipping.pileup_legacy(
+    stack = cooltools.api.snipping._pileup(
         windows, snipper.select, snipper.snip, map=map
     )
 
@@ -274,7 +283,7 @@ def test_ondiag_pileup_legacy_without_expected(request):
     assert np.all(np.isnan(stack[:, :, 1]))
 
 
-def test_offdiag_pileup_legacy_with_expected(request):
+def test_offdiag__pileup_with_expected(request):
     """
     Test the snipping on matrix:
     """
@@ -302,11 +311,11 @@ def test_offdiag_pileup_legacy_with_expected(request):
         windows = pd.merge(
             windows1, windows2, left_index=True, right_index=True, suffixes=("1", "2")
         )
-        windows = cooltools.api.snipping.assign_regions(windows, view_df).reset_index(
+        windows = cooltools.lib.common.assign_view_auto(windows, view_df).reset_index(
             drop=True
         )
 
-        stack = cooltools.api.snipping.pileup_legacy(
+        stack = cooltools.api.snipping._pileup(
             windows, snipper.select, snipper.snip, map=map
         )
 
@@ -324,11 +333,11 @@ def test_offdiag_pileup_legacy_with_expected(request):
         windows = pd.merge(
             windows1, windows2, left_index=True, right_index=True, suffixes=("1", "2")
         )
-        windows = cooltools.api.snipping.assign_regions(windows, view_df).reset_index(
+        windows = cooltools.lib.common.assign_view_auto(windows, view_df).reset_index(
             drop=True
         )
 
-        stack = cooltools.api.snipping.pileup_legacy(
+        stack = cooltools.api.snipping._pileup(
             windows, snipper.select, snipper.snip, map=map
         )
 
@@ -336,7 +345,7 @@ def test_offdiag_pileup_legacy_with_expected(request):
         assert np.all(np.isnan(stack[:, :, 1]))
 
 
-def test_offdiag_pileup_legacy_without_expected(request):
+def test_offdiag__pileup_without_expected(request):
     """
     Test the snipping on matrix:
     """
@@ -357,12 +366,12 @@ def test_offdiag_pileup_legacy_without_expected(request):
     windows = pd.merge(
         windows1, windows2, left_index=True, right_index=True, suffixes=("1", "2")
     )
-    windows = cooltools.api.snipping.assign_regions(windows, view_df).reset_index(
+    windows = cooltools.lib.common.assign_view_auto(windows, view_df).reset_index(
         drop=True
     )
 
     snipper = cooltools.api.snipping.CoolerSnipper(clr, view_df=view_df, min_diag=None)
-    stack = cooltools.api.snipping.pileup_legacy(
+    stack = cooltools.api.snipping._pileup(
         windows, snipper.select, snipper.snip, map=map
     )
 
@@ -380,11 +389,11 @@ def test_offdiag_pileup_legacy_without_expected(request):
     windows = pd.merge(
         windows1, windows2, left_index=True, right_index=True, suffixes=("1", "2")
     )
-    windows = cooltools.api.snipping.assign_regions(windows, view_df).reset_index(
+    windows = cooltools.lib.common.assign_view_auto(windows, view_df).reset_index(
         drop=True
     )
 
-    stack = cooltools.api.snipping.pileup_legacy(
+    stack = cooltools.api.snipping._pileup(
         windows, snipper.select, snipper.snip, map=map
     )
 
